import * as d3 from 'd3';

function renderData(dataElements) {
  let width = 800;
  let height = width;

  let data = dataElements.map((o) => ({
    sepalLength: +o.sepalLength,
    sepalWidth: +o.sepalWidth,
    petalLength: +o.petalLength,
    petalWidth: +o.petalWidth,
    variety: o.variety,
  }));

  let clmns = dataElements.columns;
  // remove variety from the matrix
  clmns.pop();
  let colors = d3.schemeCategory10;

  let margin = {
    top: 45,
    right: 20,
    bottom: 35,
    left: 45,
  };
  let padding = 25;
  let opacity = 0.75;

  let x = clmns;
  let xScaleType = d3.scaleLinear;

  let y = clmns;
  let yScaleType = d3.scaleLinear;
  
  // function that returns a variety from the data
  let z = (d) => d.variety;
  let domain;

  // calculate values, assign columns' names to accessors
  const X = d3.map(x, (x) =>
    d3.map(
      data,
      typeof x === 'function' ? x : (d) => d[x]
    )
  );
  const Y = d3.map(y, (y) =>
    d3.map(
      data,
      typeof y === 'function' ? y : (d) => d[y]
    )
  );
  const Z = d3.map(data, z);

  // get default domain
  if (domain === undefined) domain = Z;
  domain = new d3.InternSet(domain);

  // compute the inner dimensions of the cells.
  const boxWidth =
    (width -
      margin.left -
      margin.right -
      (X.length - 1) * padding) /
    X.length;
  const boxHeight =
    (height -
      margin.top -
      margin.bottom -
      (Y.length - 1) * padding) /
    Y.length;

  // construct scales and axes.
  const xScale = X.map((X) =>
    xScaleType(d3.extent(X), [0, boxWidth])
  );
  const yScale = Y.map((Y) =>
    yScaleType(d3.extent(Y), [boxHeight, 0])
  );
  const zScale = d3.scaleOrdinal(domain, colors);

  const xAxis = d3
    .axisBottom()
    .ticks(boxWidth / 50);
  const yAxis = d3
    .axisLeft()
    .ticks(boxHeight / 35);

  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width)
    .attr('height', height)
    .attr('viewBox', [
      -margin.left,
      -margin.top,
      width,
      height,
    ])
    .attr(
      'style',
      'max-width: 100%; height: auto; '
    );

  svg
    .append('g')
    .selectAll('g')
    .data(yScale)
    .join('g')
    .attr(
      'transform',
      (d, i) =>
        `translate(0,${
          i * (boxHeight + padding)
        })`
    )
    .each(function (d) {
      return d3.select(this).call(yAxis.scale(d));
    })
    .call((g) => g.select('.domain').remove())
    .call((g) =>
      g
        .selectAll('.tick line')
        .clone()
        .attr(
          'x2',
          width - margin.left - margin.right
        )
        .attr('stroke-opacity', 0.15)
    );

  svg
    .append('g')
    .selectAll('g')
    .data(xScale)
    .join('g')
    .attr(
      'transform',
      (d, i) => `translate(
            ${i * (boxWidth + padding)},
            ${
              height - margin.bottom - margin.top
            })`
    )
    .each(function (xScale) {
      return d3
        .select(this)
        .call(xAxis.scale(xScale));
    })
    .call((g) => g.select('.domain').remove())
    .call((g) =>
      g
        .selectAll('.tick line')
        .clone()
        .attr(
          'y2',
          -height + margin.top + margin.bottom
        )
        .attr('stroke-opacity', 0.2)
    );

  // add matrix
  const box = svg
    .append('g')
    .selectAll('g')
    .data(
      d3.cross(
        d3.range(X.length),
        d3.range(Y.length)
      )
    )
    .join('g')
    .attr('fill-opacity', opacity)
    .attr(
      'transform',
      ([i, j]) => `translate(
            ${i * (boxWidth + padding)},
            ${j * (boxHeight + padding)})`
    );

  // add border boxes around each scatterplot
  box
    .append('rect')
    .attr('fill', 'none')
    .attr('stroke', 'currentColor')
    .attr('width', boxWidth)
    .attr('height', boxHeight);

  // skip data that is not present in the domain of z.
  const filteredData = d3
    .range(Z.length)
    .filter((fd) => domain.has(Z[fd]));

  // add dots of data for each of the scatterplots
  box.each(function ([x, y]) {
    d3.select(this)
      .selectAll('circle')
      .data(
        filteredData.filter(
          (fd) =>
            !isNaN(X[x][fd]) && !isNaN(Y[y][fd])
        )
      )
      .join('circle')
      .attr('r', 3)
      .attr('cx', (d) => xScale[x](X[x][d]))
      .attr('cy', (d) => yScale[y](Y[y][d]))
      .attr('fill', (d) => {
        console.log(zScale(Z[d]));
        return zScale(Z[d]);
      });
  });

  if (x === y)
    svg
      .append('g')
      .attr('font-size', 12)
      .attr('font-weight', 'bold')
      .selectAll('text')
      .data(x)
      .join('text')
      .attr(
        'transform',
        (d, i) => `translate(
            ${i * (boxWidth + padding)},
            ${i * (boxHeight + padding)-38})`
      )
      .attr('x', padding / 2)
      .attr('y', padding / 2)
      .attr('dy', '1.5em')
      .text((d) => d);

  svg
    .append('text')
    .attr('fill', 'black')
    .attr('x', width /3.5)
    .attr('y', '-20')
    .style('font-size', '18px')
    .text('Measurements of Iris Flowers');

  return Object.assign(svg.node(), {
    scales: { color: zScale },
  });
}

d3.csv(
  'https://gist.githubusercontent.com/olga-kondr/f5e843380c653c7d4f7c0b037075b582/raw/0b0056514c32f1274c99c1d17faca0167dc2ee75/iris'
).then((data) => renderData(data));
